package com.changhu.module.management.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.changhu.common.annotation.UserType;
import com.changhu.common.enums.ResultCode;
import com.changhu.common.exception.MessageException;
import com.changhu.common.utils.UserUtil;
import com.changhu.module.management.mapper.ManagementSecurityUnitUserMapper;
import com.changhu.module.management.pojo.entity.ManagementSecurityUnitUser;
import com.changhu.module.management.pojo.params.ManagementSecurityUnitUserSaveOrUpdateParams;
import com.changhu.module.management.pojo.queryParams.ManagementSecurityUnitUserPagerQueryParams;
import com.changhu.module.management.pojo.vo.ManagementSecurityUnitUserPagerVo;
import com.changhu.module.management.service.ManagementSecurityUnitUserService;
import com.changhu.support.mybatisplus.pojo.entity.BaseEntity;
import com.changhu.support.mybatisplus.pojo.params.PageParams;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * management_security_unit_user (后台-保安单位用户表)  服务实现类
 * author: luozhun
 * desc 由groovy脚本自动生成
 */
@Service
public class ManagementSecurityUnitUserServiceImpl extends ServiceImpl<ManagementSecurityUnitUserMapper, ManagementSecurityUnitUser> implements ManagementSecurityUnitUserService {

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveOrUpdate(ManagementSecurityUnitUserSaveOrUpdateParams saveOrUpdateParams) {
        //查看手机号是否已存在
        boolean exists = this.lambdaQuery()
                .ne(saveOrUpdateParams.getSnowFlakeId() != null, BaseEntity::getSnowFlakeId, saveOrUpdateParams.getSnowFlakeId())
                .eq(ManagementSecurityUnitUser::getTelephone, saveOrUpdateParams.getTelephone())
                .exists();
        if (exists) {
            throw new MessageException("该手机号已存在");
        }

        ManagementSecurityUnitUser managementSecurityUnitUser = BeanUtil.copyProperties(saveOrUpdateParams, ManagementSecurityUnitUser.class);
        //新增 需要补全一些信息
        if (managementSecurityUnitUser.getSnowFlakeId() == null) {
            String account = RandomUtil.randomString(6);
            boolean accExits = this.lambdaQuery()
                    .eq(ManagementSecurityUnitUser::getAccount, account)
                    .exists();
            if (accExits) {
                throw new MessageException("生成的账号已存在 请重新提交生成");
            }
            //生成账号
            managementSecurityUnitUser.setAccount(account);
            if (!UserType.MANAGEMENT_SECURITY.equals(UserUtil.getUserType())) {
                throw new MessageException("客户端类型不对");
            }
            //补全单位
            managementSecurityUnitUser.setSecurityUnitId(UserUtil.getUnitId());
            //生成密码
            String passWordEncrypt = UserUtil.passWordEncrypt(UserUtil.DEFAULT_PASSWORD);
            List<String> saltAndPassWord = StrUtil.split(passWordEncrypt, "$$");
            managementSecurityUnitUser.setSalt(saltAndPassWord.get(0));
            managementSecurityUnitUser.setPassword(saltAndPassWord.get(1));
        }
        this.saveOrUpdate(managementSecurityUnitUser);
    }

    @Override
    public Page<ManagementSecurityUnitUserPagerVo> pager(PageParams<ManagementSecurityUnitUserPagerQueryParams, ManagementSecurityUnitUserPagerVo> queryParams) {
        return baseMapper.pager(queryParams.getPage(), queryParams.getParams());
    }

    @Override
    public void deleteById(Long managementSecurityUnitUserId) {
        Long unitId = UserUtil.getUnitId();
        Long l = this.lambdaQuery()
                .eq(BaseEntity::getSnowFlakeId, managementSecurityUnitUserId)
                .oneOpt()
                .map(ManagementSecurityUnitUser::getSecurityUnitId)
                .orElseThrow(() -> new MessageException(ResultCode.USER_NOT_FOUND));
        if (!unitId.equals(l)) {
            throw new MessageException("单位不匹配,无权操作");
        }
        boolean b = this.removeById(managementSecurityUnitUserId);
        if (!b) {
            throw new MessageException();
        }
    }
}
