package com.changhu.module.management.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.changhu.common.enums.ResultCode;
import com.changhu.common.exception.MessageException;
import com.changhu.common.utils.UserUtil;
import com.changhu.module.management.mapper.ManagementSuperUserMapper;
import com.changhu.module.management.pojo.entity.ManagementSuperUser;
import com.changhu.module.management.pojo.params.ManagementSuperUserSaveOrUpdateParams;
import com.changhu.module.management.service.ManagementSuperUserService;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * management_super_user (后台-超级后台)  服务实现类
 * author: luozhun
 * desc 由groovy脚本自动生成
 */
@Service
public class ManagementSuperUserServiceImpl extends ServiceImpl<ManagementSuperUserMapper, ManagementSuperUser> implements ManagementSuperUserService {

    @Override
    public void saveOrUpdate(ManagementSuperUserSaveOrUpdateParams params) {
        //查看手机号是否存在
        boolean exists = this.lambdaQuery()
                .eq(ManagementSuperUser::getTelephone, params.getTelephone())
                .exists();
        if (exists) {
            throw new MessageException("手机号已注册");
        }

        //构建用户
        ManagementSuperUser managementSuperUser = ManagementSuperUser.builder()
                .snowFlakeId(params.getSnowFlakeId())
                .name(params.getName())
                .telephone(params.getTelephone())
                .build();

        //如果是新增需要初始化密码
        if (managementSuperUser.getSnowFlakeId() == null) {
            //生成加密密码: salt$$uuid
            String passWordEncrypt = UserUtil.passWordEncrypt(UserUtil.DEFAULT_PASSWORD);
            List<String> saltAndPassWord = StrUtil.split(passWordEncrypt, "$$");
            if (CollUtil.size(saltAndPassWord) != 2) {
                throw new MessageException(ResultCode.ERROR);
            }
            managementSuperUser.setSalt(saltAndPassWord.get(0));
            managementSuperUser.setPassword(saltAndPassWord.get(1));
        }

        //保存到数据库
        this.saveOrUpdate(managementSuperUser);
    }
}
